@@ -1,6 +1,6 @@
 /* ============================================================
 * QuiteRSS is a open-source cross-platform RSS/Atom news feeds reader
-* Copyright (C) 2011-2013 QuiteRSS Team <quiterssteam@gmail.com>
+* Copyright (C) 2011-2014 QuiteRSS Team <quiterssteam@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,9 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ============================================================ */
 #include "parseobject.h"
-#include "db_func.h"
-#include "rsslisting.h"
+
+#include "mainapplication.h"
+#include "database.h"
 #include "VersionNo.h"
 
 #include <QDebug>
@@ -29,30 +30,31 @@
 #endif
 
 ParseObject::ParseObject(QObject *parent)
-  : QObject(0)
+  : QObject(parent)
   , currentFeedId_(0)
 {
   setObjectName("parseObject_");
 
-  QObject *parent_ = parent;
-  while(parent_->parent()) {
-    parent_ = parent_->parent();
-  }
-  rssl_ = qobject_cast<RSSListing*>(parent_);
+  db_ = Database::connection("secondConnection");
 
-  parseTimer_ = new QTimer();
+  parseTimer_ = new QTimer(this);
   parseTimer_->setSingleShot(true);
-  parseTimer_->setInterval(10);
-  connect(parseTimer_, SIGNAL(timeout()), this, SLOT(getQueuedXml()));
+  parseTimer_->setInterval(50);
+  connect(parseTimer_, SIGNAL(timeout()), this, SLOT(getQueuedXml()),
+          Qt::QueuedConnection);
 
   connect(this, SIGNAL(signalReadyParse(QByteArray,int,QDateTime,QString)),
           SLOT(slotParse(QByteArray,int,QDateTime,QString)));
 }
 
+ParseObject::~ParseObject()
+{
+}
+
 /** @brief Queueing xml-data
  *----------------------------------------------------------------------------*/
-void ParseObject::parseXml(const QByteArray &data, const int &feedId,
-                           const QDateTime &dtReply, const QString &codecName)
+void ParseObject::parseXml(QByteArray data, int feedId,
+                           QDateTime dtReply, QString codecName)
 {
   idsQueue_.enqueue(feedId);
   xmlsQueue_.enqueue(data);
@@ -89,8 +91,8 @@ void ParseObject::getQueuedXml()
 void ParseObject::slotParse(const QByteArray &xmlData, const int &feedId,
                             const QDateTime &dtReply, const QString &codecName)
 {
-  if (!rssl_->lastFeedPath_.isEmpty()) {
-    QFile file(rssl_->lastFeedPath_ + QDir::separator() + "lastfeed.dat");
+  if (mainApp->isSaveDataLastFeed()) {
+    QFile file(mainApp->dataDir()  + "/lastfeed.dat");
     file.open(QIODevice::WriteOnly);
     file.write(xmlData);
     file.close();
@@ -98,11 +100,13 @@ void ParseObject::slotParse(const QByteArray &xmlData, const int &feedId,
 
   qDebug() << "=================== parseXml:start ============================";
 
+  db_.transaction();
+
   // extract feed id and duplicate news mode from feed table
   parseFeedId_ = feedId;
   QString feedUrl;
   duplicateNewsMode_ = false;
-  QSqlQuery q;
+  QSqlQuery q(db_);
   q.setForwardOnly(true);
   q.exec(QString("SELECT duplicateNewsMode, xmlUrl FROM feeds WHERE id=='%1'").arg(parseFeedId_));
   if (q.first()) {
@@ -112,8 +116,9 @@ void ParseObject::slotParse(const QByteArray &xmlData, const int &feedId,
 
   // id not found (ex. feed deleted while updating)
   if (feedUrl.isEmpty()) {
-    qDebug() << QString("Feed with id = '%1' not found").arg(parseFeedId_);
-    emit feedUpdated(parseFeedId_, false, 0, "0");
+    qWarning() << QString("Feed with id = '%1' not found").arg(parseFeedId_);
+    emit signalFinishUpdate(parseFeedId_, false, 0, "0");
+    db_.commit();
     return;
   }
 
@@ -139,12 +144,12 @@ void ParseObject::slotParse(const QByteArray &xmlData, const int &feedId,
   }
   if (pos > -1) {
     QString codecNameT = rx.cap(1);
-    qDebug() << "Codec name:" << codecNameT;
+    qDebug() << "Codec name (1):" << codecNameT;
     QTextCodec *codec = QTextCodec::codecForName(codecNameT.toUtf8());
     if (codec) {
-      qDebug() << "Codec found 1";
       convertData = codec->toUnicode(xmlData);
     } else {
+      qWarning() << "Codec not found (1): " << codecNameT << feedUrl;
       if (codecNameT.contains("us-ascii", Qt::CaseInsensitive)) {
         QString str(xmlData);
         convertData = str.remove(rx.cap(0)+"\"");
@@ -152,12 +157,13 @@ void ParseObject::slotParse(const QByteArray &xmlData, const int &feedId,
     }
   } else {
     if (!codecName.isEmpty()) {
-      qDebug() << "Codec name:" << codecName;
+      qDebug() << "Codec name (2):" << codecName;
       QTextCodec *codec = QTextCodec::codecForName(codecName.toUtf8());
       if (codec) {
-        qDebug() << "Codec found 2";
         convertData = codec->toUnicode(xmlData);
         codecOk = true;
+      } else {
+        qWarning() << "Codec not found (2): " << codecName << feedUrl;
       }
     }
     if (!codecOk) {
@@ -168,8 +174,7 @@ void ParseObject::slotParse(const QByteArray &xmlData, const int &feedId,
       foreach (QString codecNameT, codecNameList) {
         QTextCodec *codec = QTextCodec::codecForName(codecNameT.toUtf8());
         if (codec && codec->canEncode(xmlData)) {
-          qDebug() << "Codec name:" << codecNameT;
-          qDebug() << "Codec found 3";
+          qDebug() << "Codec name (3):" << codecNameT;
           convertData = codec->toUnicode(xmlData);
           codecOk = true;
           break;
@@ -182,8 +187,9 @@ void ParseObject::slotParse(const QByteArray &xmlData, const int &feedId,
   }
 
   if (!doc.setContent(convertData, false, &errorStr, &errorLine, &errorColumn)) {
-    qDebug() << QString("Parse data error: line %1, column %2: %3").
-                arg(errorLine).arg(errorColumn).arg(errorStr);
+    qWarning() << QString("Parse data error (2): url %1, id %2, line %3, column %4: %5").
+                  arg(feedUrl).arg(parseFeedId_).
+                  arg(errorLine).arg(errorColumn).arg(errorStr);
   } else {
     QDomElement rootElem = doc.documentElement();
     feedType = rootElem.tagName();
@@ -208,13 +214,14 @@ void ParseObject::slotParse(const QByteArray &xmlData, const int &feedId,
 
   int newCount = 0;
   if (feedChanged_) {
-    setUserFilter(parseFeedId_);
+    runUserFilter(parseFeedId_);
     newCount = recountFeedCounts(parseFeedId_, feedUrl, updated, lastBuildDate);
   }
 
   q.finish();
+  db_.commit();
 
-  emit feedUpdated(parseFeedId_, feedChanged_, newCount, "0");
+  emit signalFinishUpdate(parseFeedId_, feedChanged_, newCount, "0");
   qDebug() << "=================== parseXml:finish ===========================";
 }
 
@@ -243,10 +250,21 @@ void ParseObject::parseAtom(const QString &feedUrl, const QDomDocument &doc)
       break;
     }
   }
+  if (feedItem.link.isEmpty()) {
+    for (int j = 0; j < linksList.size(); j++) {
+        if (!(linksList.at(j).toElement().attribute("rel") == "self")) {
+          feedItem.link = linksList.at(j).toElement().attribute("href");
+          break;
+        }
+    }
+  }
+  if (feedItem.linkBase.isEmpty() && !QUrl(feedItem.link).host().isEmpty())
+    feedItem.linkBase = QUrl(feedItem.link).scheme() %  "://" % QUrl(feedItem.link).host();
   if (QUrl(feedItem.link).host().isEmpty())
     feedItem.link = feedItem.linkBase + feedItem.link;
+  feedItem.link = toPlainText(feedItem.link);
 
-  QSqlQuery q;
+  QSqlQuery q(db_);
   q.setForwardOnly(true);
   QString qStr ("UPDATE feeds "
                 "SET title=?, description=?, htmlUrl=?, "
@@ -305,14 +323,25 @@ void ParseObject::parseAtom(const QString &feedUrl, const QDomDocument &doc)
         if (linksList.at(j).toElement().attribute("rel") == "replies")
           newsItem.comments = linksList.at(j).toElement().attribute("href");
       } else if (newsItem.linkAlternate.isEmpty()) {
-        if (!(linksList.at(j).toElement().attribute("rel") == "self"))
+        if (linksList.at(j).toElement().attribute("rel") == "alternate")
           newsItem.linkAlternate = linksList.at(j).toElement().attribute("href");
       }
     }
+    for (int j = 0; j < linksList.size(); j++) {
+      if (newsItem.linkAlternate.isEmpty()) {
+        if (!(linksList.at(j).toElement().attribute("rel") == "self")) {
+          newsItem.linkAlternate = linksList.at(j).toElement().attribute("href");
+          break;
+        }
+      }
+    }
+
     if (!newsItem.link.isEmpty() && QUrl(newsItem.link).host().isEmpty())
       newsItem.link = feedItem.linkBase + newsItem.link;
+    newsItem.link = toPlainText(newsItem.link);
     if (!newsItem.linkAlternate.isEmpty() && QUrl(newsItem.linkAlternate).host().isEmpty())
       newsItem.linkAlternate = feedItem.linkBase + newsItem.linkAlternate;
+    newsItem.linkAlternate = toPlainText(newsItem.linkAlternate);
 
     addAtomNewsIntoBase(newsItem);
   }
@@ -321,7 +350,7 @@ void ParseObject::parseAtom(const QString &feedUrl, const QDomDocument &doc)
 void ParseObject::addAtomNewsIntoBase(NewsItemStruct &newsItem)
 {
   // search news duplicates in base
-  QSqlQuery q;
+  QSqlQuery q(db_);
   q.setForwardOnly(true);
   QString qStr;
   qDebug() << "atomId:" << newsItem.id;
@@ -366,8 +395,8 @@ void ParseObject::addAtomNewsIntoBase(NewsItemStruct &newsItem)
     // if duplicates not found, add news into base
     if (!q.first()) {
       bool read = false;
-      if (rssl_->markIdenticalNewsRead_) {
-        q.prepare("SELECT * FROM news WHERE feedId!=:id AND title LIKE :title");
+      if (mainApp->mainWindow()->markIdenticalNewsRead_) {
+        q.prepare("SELECT id FROM news WHERE title LIKE :title AND feedId!=:id");
         q.bindValue(":id", parseFeedId_);
         q.bindValue(":title", newsItem.title);
         q.exec();
@@ -402,7 +431,10 @@ void ParseObject::addAtomNewsIntoBase(NewsItemStruct &newsItem)
       q.addBindValue(newsItem.eLength);
       q.addBindValue(read ? 0 : 1);
       q.addBindValue(read ? 2 : 0);
-      q.exec();
+      if (!q.exec()) {
+        qWarning() << __PRETTY_FUNCTION__ << __LINE__
+                   << "q.lastError(): " << q.lastError().text();
+      }
       qDebug() << "q.exec(" << q.lastQuery() << ")";
       qDebug() << "       " << parseFeedId_;
       qDebug() << "       " << newsItem.description;
@@ -434,7 +466,7 @@ void ParseObject::parseRss(const QString &feedUrl, const QDomDocument &doc)
 
   feedItem.title = toPlainText(channel.namedItem("title").toElement().text());
   feedItem.description = channel.namedItem("description").toElement().text();
-  feedItem.link = channel.namedItem("link").toElement().text();
+  feedItem.link = toPlainText(channel.namedItem("link").toElement().text());
   feedItem.updated = channel.namedItem("pubDate").toElement().text();
   if (feedItem.updated.isEmpty())
     feedItem.updated = channel.namedItem("pubdate").toElement().text();
@@ -442,7 +474,7 @@ void ParseObject::parseRss(const QString &feedUrl, const QDomDocument &doc)
   feedItem.author = toPlainText(channel.namedItem("author").toElement().text());
   feedItem.language = channel.namedItem("language").toElement().text();
 
-  QSqlQuery q;
+  QSqlQuery q(db_);
   q.setForwardOnly(true);
   QString qStr("UPDATE feeds "
                "SET title=?, description=?, htmlUrl=?, "
@@ -466,11 +498,17 @@ void ParseObject::parseRss(const QString &feedUrl, const QDomDocument &doc)
     newsItem.updated = newsList.item(i).namedItem("pubDate").toElement().text();
     if (newsItem.updated.isEmpty())
       newsItem.updated = newsList.item(i).namedItem("pubdate").toElement().text();
+    if (newsItem.updated.isEmpty())
+      newsItem.updated = newsList.item(i).namedItem("dc:date").toElement().text();
     newsItem.updated = parseDate(newsItem.updated, feedUrl);
     newsItem.author = toPlainText(newsList.item(i).namedItem("author").toElement().text());
     if (newsItem.author.isEmpty())
       newsItem.author = toPlainText(newsList.item(i).namedItem("dc:creator").toElement().text());
-    newsItem.link = newsList.item(i).namedItem("link").toElement().text().simplified();
+    newsItem.link = toPlainText(newsList.item(i).namedItem("link").toElement().text());
+    if (newsItem.link.isEmpty()) {
+      if (newsList.item(i).namedItem("guid").toElement().attribute("isPermaLink") == "true")
+        newsItem.link = newsItem.id;
+    }
     newsItem.description = newsList.item(i).namedItem("description").toElement().text();
     newsItem.content = newsList.item(i).namedItem("content:encoded").toElement().text();
     QDomNodeList categoryElem = newsList.item(i).toElement().elementsByTagName("category");
@@ -484,6 +522,14 @@ void ParseObject::parseRss(const QString &feedUrl, const QDomDocument &doc)
     newsItem.eType = enclosureElem.attribute("type");
     newsItem.eLength = enclosureElem.attribute("length");
 
+    if (newsItem.title.isEmpty()) {
+      newsItem.title = toPlainText(newsItem.description);
+      if (newsItem.title.size() > 50) {
+        newsItem.title.resize(50);
+        newsItem.title = newsItem.title % "...";
+      }
+    }
+
     addRssNewsIntoBase(newsItem);
   }
 }
@@ -491,7 +537,7 @@ void ParseObject::parseRss(const QString &feedUrl, const QDomDocument &doc)
 void ParseObject::addRssNewsIntoBase(NewsItemStruct &newsItem)
 {
   // search news duplicates in base
-  QSqlQuery q;
+  QSqlQuery q(db_);
   q.setForwardOnly(true);
   QString qStr;
   QString qStr1;
@@ -534,8 +580,8 @@ void ParseObject::addRssNewsIntoBase(NewsItemStruct &newsItem)
     // if duplicates not found, add news into base
     if (!q.first()) {
       bool read = false;
-      if (rssl_->markIdenticalNewsRead_) {
-        q.prepare("SELECT * FROM news WHERE feedId!=:id AND title LIKE :title");
+      if (mainApp->mainWindow()->markIdenticalNewsRead_) {
+        q.prepare("SELECT id FROM news WHERE title LIKE :title AND feedId!=:id");
         q.bindValue(":id", parseFeedId_);
         q.bindValue(":title", newsItem.title);
         q.exec();
@@ -566,7 +612,10 @@ void ParseObject::addRssNewsIntoBase(NewsItemStruct &newsItem)
       q.addBindValue(newsItem.eLength);
       q.addBindValue(read ? 0 : 1);
       q.addBindValue(read ? 2 : 0);
-      q.exec();
+      if (!q.exec()) {
+        qWarning() << __PRETTY_FUNCTION__ << __LINE__
+                   << "q.lastError(): " << q.lastError().text();
+      }
       qDebug() << "q.exec(" << q.lastQuery() << ")";
       qDebug() << "       " << parseFeedId_;
       qDebug() << "       " << newsItem.description;
@@ -590,7 +639,7 @@ void ParseObject::addRssNewsIntoBase(NewsItemStruct &newsItem)
 
 QString ParseObject::toPlainText(const QString &text)
 {
-  return QTextDocumentFragment::fromHtml(text.simplified()).toPlainText();
+  return QTextDocumentFragment::fromHtml(text).toPlainText().simplified();
 }
 
 /** @brief Date/time string parsing
@@ -603,6 +652,10 @@ QString ParseObject::parseDate(const QString &dateString, const QString &urlStri
 
   if (dateString.isEmpty()) return QString();
 
+  QDateTime dtLocalTime = QDateTime::currentDateTime();
+  QDateTime dtUTC = QDateTime(dtLocalTime.date(), dtLocalTime.time(), Qt::UTC);
+  int nTimeShift = dtLocalTime.secsTo(dtUTC)/3600;
+
   QString ds = dateString.simplified();
   QLocale locale(QLocale::C);
 
@@ -613,46 +666,55 @@ QString ParseObject::parseDate(const QString &dateString, const QString &urlStri
   for (int i = 0; i < 2; i++, locale = QLocale::system()) {
     temp     = ds.left(23);
     timeZone = ds.mid(temp.length(), 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "yyyy-MM-ddTHH:mm:ss.z");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
     temp     = ds.left(19);
     timeZone = ds.mid(temp.length(), 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "yyyy-MM-ddTHH:mm:ss");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
     temp = ds.left(23);
     timeZone = ds.mid(temp.length()+1, 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "yyyy-MM-dd HH:mm:ss.z");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
     temp = ds.left(19);
     timeZone = ds.mid(temp.length()+1, 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "yyyy-MM-dd HH:mm:ss");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
     temp = ds.left(20);
     timeZone = ds.mid(temp.length()+1, 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "dd MMM yyyy HH:mm:ss");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
     temp = ds.left(19);
     timeZone = ds.mid(temp.length()+1, 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "d MMM yyyy HH:mm:ss");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
     temp = ds.left(11);
     timeZone = ds.mid(temp.length()+1, 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "dd MMM yyyy");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
     temp = ds.left(10);
     timeZone = ds.mid(temp.length()+1, 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "d MMM yyyy");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
     temp = ds.left(10);
     timeZone = ds.mid(temp.length(), 3);
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "yyyy-MM-dd");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
 
@@ -665,6 +727,7 @@ QString ParseObject::parseDate(const QString &dateString, const QString &urlStri
     else temp2.insert(7, "20");
     temp = temp2.left(20);
     timeZone = ds.mid(temp.length()+1-2, 3);  // "-2", cause 2 symbols inserted
+    if (timeZone.isEmpty()) timeZone = QString::number(nTimeShift);
     dt = locale.toDateTime(temp, "dd MMM yyyy HH:mm:ss");
     if (dt.isValid()) return locale.toString(dt.addSecs(timeZone.toInt() * -3600), "yyyy-MM-ddTHH:mm:ss");
   }
@@ -673,6 +736,279 @@ QString ParseObject::parseDate(const QString &dateString, const QString &urlStri
   return QString();
 }
 
+/** @brief Apply user filters
+ * @param feedId - Feed Id
+ * @param filterId - Id of particular filter
+ *---------------------------------------------------------------------------*/
+void ParseObject::runUserFilter(int feedId, int filterId)
+{
+  QSqlQuery q(db_);
+  bool onlyNew = true;
+
+  if (filterId != -1) {
+    onlyNew = false;
+    q.exec(QString("SELECT enable, type FROM filters WHERE id='%1' AND feeds LIKE '%,%2,%'").
+           arg(filterId).arg(feedId));
+  } else {
+    q.exec(QString("SELECT enable, type, id FROM filters WHERE feeds LIKE '%,%1,%' ORDER BY num").
+           arg(feedId));
+  }
+
+  while (q.next()) {
+    if ((q.value(0).toInt() == 0) && onlyNew) continue;
+
+    if (onlyNew)
+      filterId = q.value(2).toInt();
+    int filterType = q.value(1).toInt();
+
+    QString qStr("UPDATE news SET");
+    QString qStr1;
+    QString qStr2;
+    QString whereStr;
+    QStringList soundList;
+    QStringList colorList;
+
+    QSqlQuery q1(db_);
+    q1.exec(QString("SELECT action, params FROM filterActions "
+                    "WHERE idFilter=='%1'").arg(filterId));
+    while (q1.next()) {
+      switch (q1.value(0).toInt()) {
+      case 0: // action -> Mark news as read
+        if (!qStr1.isNull()) qStr1.append(",");
+        qStr1.append(" new=0, read=2");
+        break;
+      case 1: // action -> Add star
+        if (!qStr1.isNull()) qStr1.append(",");
+        qStr1.append(" starred=1");
+        break;
+      case 2: // action -> Delete
+        if (!qStr1.isNull()) qStr1.append(",");
+        qStr1.append(" new=0, read=2, deleted=1, ");
+        qStr1.append(QString("deleteDate='%1'").
+                     arg(QDateTime::currentDateTime().toString(Qt::ISODate)));
+        break;
+      case 3: // action -> Add Label
+        qStr2.append(QString("%1,").arg(q1.value(1).toInt()));
+        break;
+      case 4: // action -> Play Sound
+        soundList.append(q1.value(1).toString());
+        break;
+      case 5: // action -> Show News in Notifier
+        colorList.append(q1.value(1).toString());
+        break;
+      }
+    }
+
+    if (!qStr2.isEmpty()) {
+      if (!qStr1.isNull()) qStr1.append(",");
+      qStr1.append(QString(" label=',%1'").arg(qStr2));
+    }
+
+    if (qStr1.isEmpty())
+      qStr.clear();
+    else
+      qStr.append(qStr1);
+
+    whereStr = QString(" WHERE feedId='%1' AND deleted=0").arg(feedId);
+
+    if (onlyNew) whereStr.append(" AND new=1");
+
+    qStr2.clear();
+    switch (filterType) {
+    case 1: // Match all conditions
+      qStr2.append("AND ");
+      break;
+    case 2: // Match any condition
+      qStr2.append("OR ");
+      break;
+    }
+
+    if ((filterType == 1) || (filterType == 2)) {
+      whereStr.append(" AND ( ");
+      qStr1.clear();
+
+      q1.exec(QString("SELECT field, condition, content FROM filterConditions "
+                      "WHERE idFilter=='%1'").arg(filterId));
+      while (q1.next()) {
+        if (!qStr1.isNull()) qStr1.append(qStr2);
+        QString content = q1.value(2).toString().replace("'", "''");
+        switch (q1.value(0).toInt()) {
+        case 0: // field -> Title
+          switch (q1.value(1).toInt()) {
+          case 0: // condition -> contains
+            qStr1.append(QString("UPPER(title) LIKE '%%1%' ").arg(content.toUpper()));
+            break;
+          case 1: // condition -> doesn't contains
+            qStr1.append(QString("UPPER(title) NOT LIKE '%%1%' ").arg(content.toUpper()));
+            break;
+          case 2: // condition -> is
+            qStr1.append(QString("UPPER(title) LIKE '%1' ").arg(content.toUpper()));
+            break;
+          case 3: // condition -> isn't
+            qStr1.append(QString("UPPER(title) NOT LIKE '%1' ").arg(content.toUpper()));
+            break;
+          case 4: // condition -> begins with
+            qStr1.append(QString("UPPER(title) LIKE '%1%' ").arg(content.toUpper()));
+            break;
+          case 5: // condition -> ends with
+            qStr1.append(QString("UPPER(title) LIKE '%%1' ").arg(content.toUpper()));
+            break;
+          case 6: // condition -> regExp
+            qStr1.append(QString("title REGEXP '%1' ").arg(content));
+            break;
+          }
+          break;
+        case 1: // field -> Description
+          switch (q1.value(1).toInt()) {
+          case 0: // condition -> contains
+            qStr1.append(QString("UPPER(description) LIKE '%%1%' ").arg(content.toUpper()));
+            break;
+          case 1: // condition -> doesn't contains
+            qStr1.append(QString("UPPER(description) NOT LIKE '%%1%' ").arg(content.toUpper()));
+            break;
+          case 2: // condition -> regExp
+            qStr1.append(QString("description REGEXP '%1' ").arg(content));
+            break;
+          }
+          break;
+        case 2: // field -> Author
+          switch (q1.value(1).toInt()) {
+          case 0: // condition -> contains
+            qStr1.append(QString("UPPER(author_name) LIKE '%%1%' ").arg(content.toUpper()));
+            break;
+          case 1: // condition -> doesn't contains
+            qStr1.append(QString("UPPER(author_name) NOT LIKE '%%1%' ").arg(content.toUpper()));
+            break;
+          case 2: // condition -> is
+            qStr1.append(QString("UPPER(author_name) LIKE '%1' ").arg(content.toUpper()));
+            break;
+          case 3: // condition -> isn't
+            qStr1.append(QString("UPPER(author_name) NOT LIKE '%1' ").arg(content.toUpper()));
+            break;
+          case 4: // condition -> regExp
+            qStr1.append(QString("author_name REGEXP '%1' ").arg(content));
+            break;
+          }
+          break;
+        case 3: // field -> Category
+          switch (q1.value(1).toInt()) {
+          case 0: // condition -> contains
+            qStr1.append(QString("UPPER(category) LIKE '%%1%' ").arg(content.toUpper()));
+            break;
+          case 1: // condition -> doesn't contains
+            qStr1.append(QString("UPPER(category) NOT LIKE '%%1%' ").arg(content.toUpper()));
+            break;
+          case 2: // condition -> is
+            qStr1.append(QString("UPPER(category) LIKE '%1' ").arg(content.toUpper()));
+            break;
+          case 3: // condition -> isn't
+            qStr1.append(QString("UPPER(category) NOT LIKE '%1' ").arg(content.toUpper()));
+            break;
+          case 4: // condition -> begins with
+            qStr1.append(QString("UPPER(category) LIKE '%1%' ").arg(content.toUpper()));
+            break;
+          case 5: // condition -> ends with
+            qStr1.append(QString("UPPER(category) LIKE '%%1' ").arg(content.toUpper()));
+            break;
+          case 6: // condition -> regExp
+            qStr1.append(QString("category REGEXP '%1' ").arg(content));
+            break;
+          }
+          break;
+        case 4: // field -> Status
+          if (q1.value(1).toInt() == 0) { // Status -> is
+            switch (q1.value(2).toInt()) {
+            case 0:
+              qStr1.append("new==1 ");
+              break;
+            case 1:
+              qStr1.append("read>=1 ");
+              break;
+            case 2:
+              qStr1.append("starred==1 ");
+              break;
+            }
+          } else { // Status -> isn't
+            switch (q1.value(2).toInt()) {
+            case 0:
+              qStr1.append("new==0 ");
+              break;
+            case 1:
+              qStr1.append("read==0 ");
+              break;
+            case 2:
+              qStr1.append("starred==0 ");
+              break;
+            }
+          }
+          break;
+        case 5: // field -> Link
+          switch (q1.value(1).toInt()) {
+          case 0: // condition -> contains
+            qStr1.append(QString("link_href LIKE '%%1%' ").arg(content));
+            break;
+          case 1: // condition -> doesn't contains
+            qStr1.append(QString("link_href NOT LIKE '%%1%' ").arg(content));
+            break;
+          case 2: // condition -> is
+            qStr1.append(QString("Ulink_href LIKE '%1' ").arg(content));
+            break;
+          case 3: // condition -> isn't
+            qStr1.append(QString("link_href NOT LIKE '%1' ").arg(content));
+            break;
+          case 4: // condition -> begins with
+            qStr1.append(QString("link_href LIKE '%1%' ").arg(content));
+            break;
+          case 5: // condition -> ends with
+            qStr1.append(QString("link_href LIKE '%%1' ").arg(content));
+            break;
+          case 6: // condition -> regExp
+            qStr1.append(QString("link_href REGEXP '%1' ").arg(content));
+            break;
+          }
+          break;
+        }
+      }
+      whereStr.append(qStr1).append(")");
+    }
+
+    if (!qStr.isEmpty()) {
+      qStr.append(whereStr);
+      if (!q1.exec(qStr)) {
+        qWarning() << __PRETTY_FUNCTION__ << __LINE__
+                   << "q.lastError(): " << q1.lastError().text();
+      }
+    }
+
+    if (!soundList.isEmpty()) {
+      qStr = "SELECT * FROM news";
+      qStr.append(whereStr);
+      if (q1.exec(qStr)) {
+        if (q1.first()) {
+          emit signalPlaySound(soundList.at(0));
+        }
+      } else {
+        qWarning() << __PRETTY_FUNCTION__ << __LINE__
+                   << "q.lastError(): " << q1.lastError().text();
+      }
+    }
+
+    if (!colorList.isEmpty()) {
+      qStr = "SELECT id FROM news";
+      qStr.append(whereStr);
+      if (q1.exec(qStr)) {
+        while (q1.next()) {
+          emit signalAddColorList(q1.value(0).toInt(), colorList.at(0));
+        }
+      } else {
+        qWarning() << __PRETTY_FUNCTION__ << __LINE__
+                   << "q.lastError(): " << q1.lastError().text();
+      }
+    }
+
+  }
+}
+
 /** @brief Update feed counts and all its parent categories
  *
  *  Update fields: unread news number,
@@ -684,7 +1020,7 @@ QString ParseObject::parseDate(const QString &dateString, const QString &urlStri
 int ParseObject::recountFeedCounts(int feedId, const QString &feedUrl,
                                    const QString &updated, const QString &lastBuildDate)
 {
-  QSqlQuery q;
+  QSqlQuery q(db_);
   q.setForwardOnly(true);
   QString qStr;
   QString htmlUrl;
@@ -698,6 +1034,7 @@ int ParseObject::recountFeedCounts(int feedId, const QString &feedUrl,
     title = q.value(2).toString();
   }
 
+  FeedCountStruct counts;
   int undeleteCount = 0;
   int unreadCount = 0;
   int newNewsCount = 0;
@@ -734,6 +1071,14 @@ int ParseObject::recountFeedCounts(int feedId, const QString &feedUrl,
 
   if ((unreadCount == unreadCountOld) && (newNewsCount == newCountOld) &&
       (undeleteCount == undeleteCountOld)) {
+    counts.feedId = feedId;
+    counts.unreadCount = unreadCount;
+    counts.newCount = newNewsCount;
+    counts.undeleteCount = undeleteCount;
+    counts.updated = updated;
+    counts.lastBuildDate = lastBuildDate;
+
+    emit feedCountsUpdate(counts);
     return 0;
   }
 
@@ -743,7 +1088,6 @@ int ParseObject::recountFeedCounts(int feedId, const QString &feedUrl,
       arg(unreadCount).arg(newNewsCount).arg(undeleteCount).arg(feedId);
   q.exec(qStr);
 
-  FeedCountStruct counts;
   counts.feedId = feedId;
   counts.unreadCount = unreadCount;
   counts.newCount = newNewsCount;
